package com.mixel.docusphere.service;

import com.mixel.docusphere.dto.UserDTO;
import com.mixel.docusphere.entity.User;
import com.mixel.docusphere.mapper.UserMapper;
import com.mixel.docusphere.repository.UserRepository;
import org.springframework.security.crypto.argon2.Argon2PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

@Service
public class UserService {

    private final UserRepository userRepository;

    private final Argon2PasswordEncoder passwordEncoder;

    // Constructor
    public UserService(UserRepository userRepository) {
        this.passwordEncoder = new Argon2PasswordEncoder(16, 32, 1, 4096, 3);
        this.userRepository = userRepository;
    }

    public List<UserDTO> findAll() {
        return userRepository.findAll().stream()
                .map(UserMapper::toDTO)
                .collect(Collectors.toList());
    }

    public Optional<UserDTO> findById(UUID id) {
        return userRepository.findById(id)
                .map(UserMapper::toDTO);
    }

    public UserDTO save(UserDTO userDTO) {
        User user = UserMapper.toEntity(userDTO);
        isPasswordAlreadySet(userDTO, user);
        return UserMapper.toDTO(userRepository.save(user));
    }

    public Optional<UserDTO> update(UUID id, UserDTO userDTO){
        Optional<User> userOptional = userRepository.findById(id);
        if (userOptional.isPresent()) {
            User user = userOptional.get();
            user.setUsername(userDTO.getUsername());
            user.setName(userDTO.getName());
            user.setEmail(userDTO.getEmail());

            isPasswordAlreadySet(userDTO, user);
            return Optional.of(UserMapper.toDTO(userRepository.save(user)));
        }
        return Optional.empty();
    }

    private void isPasswordAlreadySet(UserDTO userDTO, User user) {
        if (userDTO.getPassword() != null && !userDTO.getPassword().isEmpty()) {
            user.setPasswordHash(passwordEncoder.encode(userDTO.getPassword()));
        }
    }

    public void deleteById(UUID id) {
        userRepository.deleteById(id);
    }
}